package com.io.yutian.elementoriginlib.command;

import com.io.yutian.elementoriginlib.command.handler.CommandHandler;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandMap;
import org.bukkit.plugin.Plugin;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SimpleCommandManager implements ICommandManager {

    protected static Map<String, Command> bukkitCommandMap = new HashMap<>();
    protected static CommandMap commandMap;

    private final Plugin plugin;
    private final String name;

    @NotNull
    private List<Class<?>> commandClasses = new ArrayList<>();
    @NotNull
    private List<CommandEntity> commands = new ArrayList<>();


    public SimpleCommandManager(Plugin plugin, String name) {
        this(plugin, name, new ArrayList<>());
    }

    public SimpleCommandManager(Plugin plugin, String name, @NotNull List<Class<?>> commandClasses) {
        this.plugin = plugin;
        this.name = name;
        this.commandClasses = commandClasses;
    }

    public void register(@NotNull Class<?> commandClass) {
        if (commandClass == null) {
            return;
        }
        CommandEntity command = CommandEntity.parseFromClass(commandClass);
        if (command == null) {
            return;
        }
        commandClasses.add(commandClass);
        commands.add(command);
    }

    public void unregisterAll() {
        commands.clear();
    }

    public void unregister(Class<?> commandClass) {
        if (commandClass == null) {
            return;
        }
        commands.remove(commandClass);
    }

    public void registerPluginCommand(@NotNull String commandName) {
        Bukkit.getPluginCommand(commandName).setExecutor(new CommandHandler(this));
    }

    public Plugin getPlugin() {
        return plugin;
    }

    @NotNull
    @Override
    public String getName() {
        return name;
    }

    @NotNull
    @Override
    public List<CommandEntity> getCommands() {
        return commands;
    }

    static {
        try {
            Class<?> c = Bukkit.getServer().getClass();
            for (Method method : c.getDeclaredMethods()) {
                if (method.getName().equals("getCommandMap")) {
                    commandMap = (CommandMap) method.invoke(Bukkit.getServer(), new Object[0]);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
